package com.usky.dxtop.framework.web.service;

import com.usky.dxtop.common.constant.Constants;
import com.usky.dxtop.common.constant.UserConstants;
import com.usky.dxtop.common.core.domain.model.LoginUser;
import com.usky.dxtop.common.core.redis.RedisCache;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.exception.user.CaptchaException;
import com.usky.dxtop.common.exception.user.CaptchaExpireException;
import com.usky.dxtop.common.exception.user.UserPasswordNotMatchException;
import com.usky.dxtop.common.utils.DateUtils;
import com.usky.dxtop.common.utils.MessageUtils;
import com.usky.dxtop.common.utils.ServletUtils;
import com.usky.dxtop.common.utils.ip.IpUtils;
import com.usky.dxtop.framework.manager.AsyncManager;
import com.usky.dxtop.framework.manager.factory.AsyncFactory;
import com.usky.dxtop.model.AccountLock;
import com.usky.dxtop.model.SysUser;
import com.usky.dxtop.service.AccountLockService;
import com.usky.dxtop.service.ISysConfigService;
import com.usky.dxtop.service.ISysUserService;
import com.usky.dxtop.service.vo.UserAccountVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Objects;
import java.util.concurrent.TimeUnit;


/**
 * 登录校验方法
 * 
 * @author yq
 */
@Component
public class SysLoginService
{
    @Autowired
    private TokenService tokenService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;
    
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private AccountLockService accountLockService;
    /**
     * 登录验证
     * 
     * @param username 用户名
     * @param password 密码
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid,Boolean captchaOnOff)
    {
        if (null == captchaOnOff){
            captchaOnOff = configService.selectCaptchaOnOff();
        }
        // 验证码开关
        if (captchaOnOff)
        {
            validateCaptcha(username, code, uuid);
        }
        // 用户验证
        Authentication authentication = null;
        try
        {
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager
                    .authenticate(new UsernamePasswordAuthenticationToken(username, password));
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                lockAccount(username);
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new CustomException(e.getMessage());
            }
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordLoginInfo(loginUser.getUser());
        // 生成token
        return tokenService.createToken(loginUser);
    }

    /**
     * 校验验证码
     * 
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        String verifyKey = Constants.CAPTCHA_CODE_KEY + uuid;
        String captcha = redisCache.getCacheObject(verifyKey);
        redisCache.deleteObject(verifyKey);
        if (captcha == null)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
            throw new CaptchaExpireException();
        }
        if (!code.equalsIgnoreCase(captcha))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
            throw new CaptchaException();
        }
    }

    /**
     * 记录登录信息
     */
    public void recordLoginInfo(SysUser user)
    {
        user.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        user.setLoginDate(DateUtils.getNowDate());
        userService.updateUserProfile(user);
    }


    /**
     * 锁定账号
     * @param userName
     */
    public void lockAccount(String userName){
        AccountLock accountLock = accountLockService.isEnableOne();
        if (null != accountLock){
            Object cacheObj = redisCache.getCacheObject(getLockAccountKey(userName));
            if (Objects.nonNull(cacheObj))
            {
                UserAccountVO userAccountVO = (UserAccountVO) cacheObj;
                userAccountVO.setUseCount(userAccountVO.getUseCount()+1);
                int i = userAccountVO.getTotalCount() - userAccountVO.getUseCount();
                if (userAccountVO.getTotalCount() > userAccountVO.getUseCount()){
                    redisCache.setCacheObject(getLockAccountKey(userName),userAccountVO);
                    throw new CustomException("密码输入错误还剩下"+i+"次机会");
                } else {
                    redisCache.setCacheObject(getLockAccountKey(userName),userAccountVO,accountLock.getLoseDate(), TimeUnit.MINUTES);
                    throw new CustomException(userAccountVO.getTotalCount()+"次机会已经用完,账号被锁定"+accountLock.getLoseDate()+"分钟");
                }
            }else {
                UserAccountVO userAccountVO = new UserAccountVO();
                userAccountVO.setTotalCount(accountLock.getLockNumber());
                userAccountVO.setUseCount(1);
                redisCache.setCacheObject(getLockAccountKey(userName),userAccountVO);
                int i = userAccountVO.getTotalCount() - 1;
                throw new CustomException("密码输入错误还剩下"+i+"次机会");
            }
        }else {
            throw new CustomException("密码输入不正确");
        }
    }

    public String getLockAccountKey(String userName){
        return UserConstants.ACCOUNT_LOCK + "_" + userName;
    }
}
