package com.usky.dxtop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.common.config.DxConfig;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.file.FileTypeUtils;
import com.usky.dxtop.common.utils.file.FileUploadUtils;
import com.usky.dxtop.common.utils.file.FileUtils;
import com.usky.dxtop.common.utils.uuid.IdUtils;
import com.usky.dxtop.mapper.SysFileMapper;
import com.usky.dxtop.model.SysFile;
import com.usky.dxtop.service.SysFileService;
import com.usky.dxtop.service.vo.SysFileQueryRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.Date;
import java.util.List;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yq
 * @since 2021-10-09
 */
@Slf4j
@Service
public class SysFileServiceImpl extends ServiceImpl<SysFileMapper, SysFile> implements SysFileService {

    public static final String FILE = "file";

    public static final String URL = "url";

    private static final String FILE_PATH = "https://smartpark.caih.com/dxfile/";

    @Override
    public List<SysFile> addFile(SysFile sysFile) {
        if (StringUtils.isBlank(sysFile.getUploadType())) {
            sysFile.setUploadType(FILE);
        }
        sysFile.setSuccess(true);
        switch (sysFile.getUploadType()) {
            case FILE:
                return sysFile.getFiles().stream().map(m -> this.uploadMultipartFile(m, sysFile)).collect(toList());
            case URL:
                return sysFile.getUrls().stream().map(m -> this.addExternalLink(m, sysFile)).collect(toList());
            default:
                break;
        }
        return null;
    }

    @Override
    public SysFile uploadMultipartFile(MultipartFile m, SysFile sysFile) {
        try {
            // 上传文件路径
            String filePath = DxConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, m);
            sysFile.setOriName(m.getOriginalFilename());
            sysFile.setFileSize(Integer.parseInt(String.valueOf(m.getSize())));
            sysFile.setFileType(m.getContentType());
            sysFile.setUrl(filePath);
            sysFile.setUrl(fileName);
            this.save(sysFile);
        } catch (Exception e) {
            sysFile.setSuccess(false);
            sysFile.setMessage(e.getMessage());
        }
        return sysFile;
    }

    @Override
    public SysFile addExternalLink(String url, SysFile sysFile) {
        try {
            String filePath = DxConfig.getUploadPath();
            String format = String.format("%s/%s.%s", filePath, System.currentTimeMillis() + "", FileTypeUtils.getFileType(url));
            File file = FileUtils.downloadFile(url, format);
            sysFile.setOriName(url.substring(url.lastIndexOf("/")+1));
            sysFile.setFileSize(Integer.parseInt(String.valueOf(file.length())));
            sysFile.setFileType(FileTypeUtils.getFileType(file));
            sysFile.setUrl(format);
            this.save(sysFile);
        }catch (Exception e){
            sysFile.setSuccess(false);
            sysFile.setMessage(e.getMessage());
        }
        return sysFile;
    }

    @Override
    public String generateBatchNo() {
        return IdUtils.randomUUID();
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW, rollbackFor = Exception.class)
    public Boolean active(String batchNo, List<String> fileIds) {
        if (CollectionUtils.isEmpty(fileIds)) {
            return false;
        }
        LambdaUpdateWrapper<SysFile> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.set(SysFile::getActiveFlag,1)
                .eq(SysFile::getBatchNo,batchNo)
                .in(SysFile::getId,fileIds);
        return this.update(updateWrapper);
    }

    @Override
    public List<SysFile> getFile(SysFileQueryRequest req) {
        LambdaQueryWrapper<SysFile> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(StringUtils.isNotBlank(req.getBatchNo()), SysFile::getBatchNo, req.getBatchNo())
                .eq(SysFile::getActiveFlag, 1)
                .in(!CollectionUtils.isEmpty(req.getIds()), SysFile::getId, req.getIds())
                .in(!CollectionUtils.isEmpty(req.getBatchNos()), SysFile::getBatchNo, req.getBatchNos());
        List<SysFile> list = this.list(queryWrapper);
        return list.stream().peek(sysFile -> {
            String url = sysFile.getUrl();
            sysFile.setUrl(String.format("%s%s",FILE_PATH,url.substring(url.lastIndexOf("/")+1)));
        }).collect(toList());
    }

    @Override
    public Integer deleteFiles(List<SysFile> sysFiles) {
        Date now = new Date();
        sysFiles = sysFiles.stream().peek(s -> {
            s.setActiveFlag(0);
            s.setExpriceAt(now);
        }).collect(toList());
        return updateBatchById(sysFiles) ? sysFiles.size() : 0;
    }

    @Override
    public Integer removeByBatchNo(String batchNo) {
        LambdaQueryWrapper<SysFile> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SysFile::getBatchNo, batchNo);
        List<SysFile> sysFileList = list(queryWrapper);
        return this.deleteFiles(sysFileList);
    }

    @Override
    public Integer clearFiles(List<SysFile> fileList) {
        if (CollectionUtils.isEmpty(fileList)){
            return 0;
        }
        fileList.forEach(sysFile -> {
            try {
                FileUtils.deleteFile(sysFile.getUrl());
            }catch (Exception e){
                log.error("删除文件失败:"+e);
            }
        });
        fileList = fileList.stream().peek(s -> {
            s.setDelFlag(1);
        }).collect(toList());
        return updateBatchById(fileList) ? fileList.size() : 0;
    }
}
